package eu.mhsl.craftattack.spawn.appliances.privateMessage;

import eu.mhsl.craftattack.spawn.Main;
import eu.mhsl.craftattack.spawn.appliance.Appliance;
import eu.mhsl.craftattack.spawn.appliance.ApplianceCommand;
import eu.mhsl.craftattack.spawn.appliances.chatMessages.ChatMessages;
import eu.mhsl.craftattack.spawn.appliances.privateMessage.commands.PrivateMessageCommand;
import eu.mhsl.craftattack.spawn.appliances.privateMessage.commands.PrivateReplyCommand;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.event.ClickEvent;
import net.kyori.adventure.text.event.HoverEvent;
import net.kyori.adventure.text.format.NamedTextColor;
import org.bukkit.Bukkit;
import org.bukkit.entity.Player;
import org.jetbrains.annotations.NotNull;

import java.util.*;
import java.util.stream.Collectors;

public class PrivateMessage extends Appliance {
    public final int targetChangeTimeoutSeconds = 30;
    public final int conversationTimeoutMinutes = 30;

    private record Conversation(UUID target, Long lastSet) {}
    private final Map<Player, List<Conversation>> replyMapping = new WeakHashMap<>();

    public void reply(Player sender, String message) {
        if(this.replyMapping.get(sender) != null) {
            List<Conversation> tooOldConversations = this.replyMapping.get(sender).stream()
                .filter(conversation -> conversation.lastSet < System.currentTimeMillis() - (conversationTimeoutMinutes*60*1000))
                .toList();
            this.replyMapping.get(sender).removeAll(tooOldConversations);
        }

        if(this.replyMapping.get(sender) == null || this.replyMapping.get(sender).isEmpty()) throw new ApplianceCommand.Error("Du führst aktuell keine Konversation.");


        Component privatePrefix = Component.text("[Privat] ", NamedTextColor.LIGHT_PURPLE);
        ChatMessages chatMessages = Main.instance().getAppliance(ChatMessages.class);

        Conversation youngestEntry = this.replyMapping.get(sender).stream()
            .max(Comparator.comparingLong(o -> o.lastSet))
            .orElse(this.replyMapping.get(sender).getLast());

        String youngestTargetName;
        if(Bukkit.getPlayer(youngestEntry.target()) == null) {
            youngestTargetName = Bukkit.getOfflinePlayer(youngestEntry.target()).getName();
        } else {
            youngestTargetName = Objects.requireNonNull(Bukkit.getPlayer(youngestEntry.target())).getName();
        }


        if(message.isBlank() && youngestTargetName != null) {
            sender.sendMessage(
                privatePrefix
                    .append(Component.text("Du schreibst aktuell mit ", NamedTextColor.GRAY))
                    .append(chatMessages.getReportablePlayerName(sender))
            );
            return;
        } else if(message.isBlank()) {
            sender.sendMessage(
                privatePrefix
                    .append(Component.text("Du schreibst aktuell mit niemandem.", NamedTextColor.GRAY))
            );
            return;
        }

        if(youngestEntry.lastSet < System.currentTimeMillis() - (targetChangeTimeoutSeconds*1000) || this.replyMapping.get(sender).size() == 1) {
            Player target = Bukkit.getPlayer(youngestEntry.target());
            if(target == null) throw new ApplianceCommand.Error("Der Spieler " + Bukkit.getOfflinePlayer(youngestEntry.target()).getName() + " ist nicht mehr verfügbar.");

            this.replyMapping.get(sender).clear();
            sendWhisper(sender, new ResolvedPmUserArguments(target, message));
            return;
        }

        List<Conversation> oldConversations = this.replyMapping.get(sender).stream()
            .filter(conversation -> conversation.lastSet < System.currentTimeMillis() - (targetChangeTimeoutSeconds*1000))
            .toList();

        Conversation youngestOldConversation = null;
        if(!oldConversations.isEmpty()) {
            youngestOldConversation = oldConversations.stream()
                .max(Comparator.comparingLong(o -> o.lastSet))
                .orElse(oldConversations.getLast());
            this.replyMapping.get(sender).removeAll(oldConversations);
        }

        List<String> playerNames = this.replyMapping.get(sender).stream()
            .map(conversation -> {
                if(Bukkit.getPlayer(conversation.target()) != null) {
                    return Objects.requireNonNull(Bukkit.getPlayer(conversation.target())).getName();
                } else {
                    return Bukkit.getOfflinePlayer(conversation.target()).getName();
                }
            })
            .distinct()
            .toList();


        sender.sendMessage("");
        sender.sendMessage(
            Component.text()
                .append(privatePrefix)
                .append(Component.text("Das Ziel für /r hat sich bei dir in den letzten ", NamedTextColor.RED))
                .append(Component.text(String.valueOf(this.targetChangeTimeoutSeconds), NamedTextColor.RED))
                .append(Component.text(" Sekunden geändert. Wer soll deine Nachricht bekommen? ", NamedTextColor.RED))
        );

        final Component[] finalComponent = {Component.text("")};

        String firstTargetName = null;
        if(youngestOldConversation != null && Bukkit.getPlayer(youngestOldConversation.target()) == null) {
            firstTargetName = Bukkit.getOfflinePlayer(youngestOldConversation.target()).getName();
        } else if(youngestOldConversation != null){
            firstTargetName = Objects.requireNonNull(Bukkit.getPlayer(youngestOldConversation.target())).getName();
        }

        if(firstTargetName != null && !playerNames.contains(firstTargetName)) {
            finalComponent[0] = finalComponent[0].append(
                Component.text("[")
                    .append(Component.text(firstTargetName, NamedTextColor.GOLD))
                    .append(Component.text("]"))
                    .clickEvent(ClickEvent.runCommand("/msg " + firstTargetName + " " + message))
                    .hoverEvent(HoverEvent.showText(Component.text("Klicke, um diesem Spieler zu schreiben.").color(NamedTextColor.GOLD))))
                .append(Component.text("  "));
        }

        playerNames.forEach(playerName -> finalComponent[0] = finalComponent[0].append(
            Component.text("[")
                .append(Component.text(playerName, NamedTextColor.GOLD))
                .append(Component.text("]"))
                .clickEvent(ClickEvent.runCommand("/msg " + playerName + " " + message))
                .hoverEvent(HoverEvent.showText(Component.text("Klicke, um diesem Spieler zu schreiben.").color(NamedTextColor.GOLD))))
            .append(Component.text("  ")));

        sender.sendMessage("");
        sender.sendMessage(finalComponent[0]);
        sender.sendMessage("");

    }

    public void sendWhisper(Player sender, ResolvedPmUserArguments userArguments) {

        if(!(this.replyMapping.get(userArguments.receiver) == null)) {
            List<Conversation> oldEntries = this.replyMapping.get(userArguments.receiver).stream()
                .filter(conversation -> conversation.target() == sender.getUniqueId())
                .toList();
            this.replyMapping.get(userArguments.receiver).removeAll(oldEntries);
        }

        Conversation newReceiverConversation = new Conversation(
            sender.getUniqueId(),
            System.currentTimeMillis()
        );

        if(this.replyMapping.get(userArguments.receiver) == null) {
            List<Conversation> receiverConversationList = new ArrayList<>();
            receiverConversationList.add(newReceiverConversation);

            this.replyMapping.put(
                userArguments.receiver,
                receiverConversationList
            );
        } else {
            this.replyMapping.get(userArguments.receiver).add(newReceiverConversation);
        }

        List<Conversation> senderConversationList = new ArrayList<>();
        senderConversationList.add(
            new Conversation(
                userArguments.receiver.getUniqueId(),
                System.currentTimeMillis()
            )
        );

        this.replyMapping.put(
            sender,
            senderConversationList
        );


        ChatMessages chatMessages = Main.instance().getAppliance(ChatMessages.class);
        Component privatePrefix = Component.text("[Privat] ", NamedTextColor.LIGHT_PURPLE);

        sender.sendMessage(
            Component.text()
                .append(privatePrefix.clickEvent(ClickEvent.suggestCommand(String.format("/msg %s ", userArguments.receiver.getName()))))
                .append(sender.displayName())
                .append(Component.text(" zu ", NamedTextColor.GRAY))
                .append(chatMessages.getReportablePlayerName(userArguments.receiver))
                .append(Component.text(" > ", NamedTextColor.GRAY))
                .append(Component.text(userArguments.message))
        );
        userArguments.receiver.sendMessage(
            Component.text()
                .append(privatePrefix.clickEvent(ClickEvent.suggestCommand(String.format("/msg %s ", sender.getName()))))
                .append(chatMessages.getReportablePlayerName(sender))
                .append(Component.text(" zu ", NamedTextColor.GRAY))
                .append(userArguments.receiver.displayName())
                .append(Component.text(" > ", NamedTextColor.GRAY))
                .append(Component.text(userArguments.message))
        );
    }

    public record ResolvedPmUserArguments(Player receiver, String message) {}
    public ResolvedPmUserArguments resolveImplicit(String[] args) {
        if(args.length < 2) throw new ApplianceCommand.Error("Es muss ein Spieler sowie eine Nachricht angegeben werden.");
        List<String> arguments = List.of(args);
        Player targetPlayer = Bukkit.getPlayer(arguments.getFirst());
        if(targetPlayer == null) throw new ApplianceCommand.Error(String.format("Der Spieler %s konnte nicht gefunden werden.", arguments.getFirst()));
        String message = arguments.stream().skip(1).collect(Collectors.joining(" "));
        return new ResolvedPmUserArguments(targetPlayer, message);
    }

    @Override
    protected @NotNull List<ApplianceCommand<?>> commands() {
        return List.of(
            new PrivateMessageCommand(),
            new PrivateReplyCommand()
        );
    }
}
